#! /bin/sh
# the next line restarts using tclsh \
exec `which tclsh` $0 ${1+"$@"}

set version "v0.2"

if {0} {
# WLAN-stats-generator for Kismet's .csv-files
# Written 2006 by Jannis Achstetter <jannis_achstetter@web.de>
# 
# Changelog:
#
# *v0.2:
#   - Added network-listing
#   - minor changes and fixes
# 
# *v0.1:
#   - Initial release
# 
# /Changelog
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
}

proc scanfiles {} {
  global verbose path listnws
  if {[catch {set list [split [glob "$path\*.csv"]]}]} {
    puts "FATAL ERROR - I didn't find any Kismet .csv-files in \"$path\""
    exit 1
  } else {
    if {$verbose} {puts "OK, I found [llength $list] .csv-files in \"$path\""}
  }

  array set types {}
  array set essids {}
  array set chans {}
  array set chancounts {}
  array set encrs {}
  array set encrcounts {}
  array set rates {}
  array set datapackets {}
  array set cryptedpackets {}
  array set weakpackets {}
  array set totalpackets {}
  array set firstseen {}
  array set lastseen {}
  array set ips {}
  array set ipcounts {}

  set netfcount 0
  foreach i [lsort $list] {
    if {$verbose} {puts -nonewline "Reading file $i ... " ; flush stdout}
    set fchan [open "$i" r]

    set fncount 0
    while {![eof $fchan]} {
      gets $fchan line
      set list [split $line \x3B]
      set bssid [lindex $list 3]

      if {$bssid != "BSSID" && $bssid != ""} {
        set nettype [lindex $list 1]
        set essid [lindex $list 2]
        set channel [lindex $list 5]
        set encr [lindex $list 7]
        set rate [lindex $list 9]

        if {[lindex [array get datapackets $bssid] 1] == ""} {array set datapackets [list $bssid 0]}
        if {[lindex [array get cryptedpackets $bssid] 1] == ""} {array set cryptedpackets [list $bssid 0]}
        if {[lindex [array get weakpackets $bssid] 1] == ""} {array set weakpackets [list $bssid 0]}
        if {[lindex [array get totalpackets $bssid] 1] == ""} {array set totalpackets [list $bssid 0]}
        set datapks [expr  [lindex $list 13] + [lindex [array get datapackets $bssid] 1]]
        set cryptpks [expr [lindex $list 14] + [lindex [array get cryptedpackets $bssid] 1]]
        set weakpks [expr  [lindex $list 15] + [lindex [array get weakpackets $bssid] 1]]
        set totalpks [expr [lindex $list 16] + [lindex [array get totalpackets $bssid] 1]]

	set firsttime [lindex $list 19]
	set lasttime [lindex $list 20]
        if {[lindex [array get firstseen $bssid] 1] == ""} {
	  array set firstseen [list $bssid $firsttime]
	}
	array set lastseen [list $bssid $lasttime]

        set ip [lindex $list 37]
        if {[string length $channel] == 1} { set channel "0$channel" }

        array set essids [list $bssid $essid]
        array set chans [list $bssid $channel]
        array set encrs [list $bssid $encr]
        array set datapackets [list $bssid $datapks]
        array set cryptedpackets [list $bssid $cryptpks]
        array set weakpackets [list $bssid $weakpks]
        array set totalpackets [list $bssid $totalpks]

        array set ips [list $bssid $ip]
        incr fncount
      }
    }
    if {$verbose} {puts "OK, found $fncount networks"}
    incr netfcount $fncount
  }

  if {$verbose} {puts ""}
  if {$listnws && $verbose} {puts "Network-list:"}
  if {$listnws} {puts "BSSID (MAC of AP) Chan ESSID\t\t\tEncr\t\t\tData\tCrypt\tWeak\tTotal\tVFTS\t\t\t\tVLTS"}
  set netcount 0
  foreach k [array names essids] {
    set channel [lindex [split [array get chans $k]] 1]
    set chancount [lindex [array get chancounts $channel] 1]
    if {$chancount == ""} { set chancount 1 } else { incr chancount }
    array set chancounts [list $channel $chancount]
    incr netcount

    # Begin LIST-code
    if {$listnws} {
      set essidshort [string range [lindex [array get essids $k] 1] 0 22]
      while {[string length $essidshort] < 22} {
        append essidshort " "
      }
      set encshort [string range [lindex [array get encrs $k] 1] 0 22]
      while {[string length $encshort] < 22} {
        append encshort " "
      }
      puts "[lindex [array get essids $k] 0] $channel   $essidshort\t$encshort\t[lindex [array get datapackets $k] 1]\t[lindex [array get cryptedpackets $k] 1]\t[lindex [array get weakpackets $k] 1]\t[lindex [array get totalpackets $k] 1]\t[lindex [array get firstseen $k] 1]\t[lindex [array get lastseen $k] 1]"
    }
    # End LIST-code
  }

  if {$listnws} {puts ""}
  if {$verbose} {puts "Total $netfcount networks - $netcount unique BSSIDs"}
  if {$verbose} {puts "\nChannel-Usage-Statistics:"}
  puts "Channel\tCount\tPerc."
  foreach l [lsort [array names chancounts]] {
    set percentage [expr ([lindex [array get chancounts $l] 1] / ($netcount + 0.0)) * 100]
    set precom [lindex [split $percentage .] 0]
    set postcom [lindex [split $percentage .] 1]
    if {[string length $precom] == 1} {
      set precom "0$precom"
    }
    set postcom [string range $postcom 0 1]
    if {[string length $postcom] < 2} {
      append postcom 0
    }
    puts "[lindex [array get chancounts $l] 0]\t[lindex [array get chancounts $l] 1]\t$precom\.$postcom%"
  }

  foreach k [array names encrs] {
    set encr [lindex [split [array get encrs $k]] 1]
    set encrcount [lindex [array get encrcounts $encr] 1]
    if {$encrcount == ""} { set encrcount 1 } else { incr encrcount }
    array set encrcounts [list $encr $encrcount]
  }

  if {$verbose} {puts -nonewline "\nEncryption-Statistics:"}
  puts "\nEncryption-name\t\t\tCount\tPerc."
  foreach l [lsort [array names encrcounts]] {
    set percentage [expr ([lindex [array get encrcounts $l] 1] / ($netcount + 0.0)) * 100]
    set precom [lindex [split $percentage .] 0]
    set postcom [lindex [split $percentage .] 1]
    if {[string length $precom] == 1} {
      set precom "0$precom"
    }
    set postcom [string range $postcom 0 1]
    if {[string length $postcom] < 2} {
      append postcom 0
    }
    if {[string length [lindex [array get encrcounts $l] 0]] < 8} {
      puts "[lindex [array get encrcounts $l] 0]\t\t\t\t[lindex [array get encrcounts $l] 1]\t$precom\.$postcom%"
    } elseif {[string length [lindex [array get encrcounts $l] 0]] < 16} {
      puts "[lindex [array get encrcounts $l] 0]\t\t\t[lindex [array get encrcounts $l] 1]\t$precom\.$postcom%"
    } elseif {[string length [lindex [array get encrcounts $l] 0]] < 24} {
      puts "[lindex [array get encrcounts $l] 0]\t\t[lindex [array get encrcounts $l] 1]\t$precom\.$postcom%"
    } else {
      puts "[lindex [array get encrcounts $l] 0]\t[lindex [array get encrcounts $l] 1]\t$precom\.$postcom%"
    }
  }

  foreach k [array names ips] {
    set ip [lindex [split [array get ips $k]] 1]
    set ip "[lindex [split $ip .] 0].[lindex [split $ip .] 1].[lindex [split $ip .] 2].XXX"
    set ipcount [lindex [array get ipcounts $ip] 1]
    if {$ipcount == ""} { set ipcount 1 } else { incr ipcount }
    array set ipcounts [list $ip $ipcount]
  }

  if {$verbose} {puts -nonewline "\nIP-Usage-Statistics:"}
  puts "\nIP-Range\t\tCount\tPerc."
  foreach l [lsort [array names ipcounts]] {
    set percentage [expr ([lindex [array get ipcounts $l] 1] / ($netcount + 0.0)) * 100]
    set precom [lindex [split $percentage .] 0]
    set postcom [lindex [split $percentage .] 1]
    if {[string length $precom] == 1} {
      set precom "0$precom"
    }
    set postcom [string range $postcom 0 1]
    if {[string length $postcom] < 2} {
      append postcom 0
    }
    if {[lindex [array get ipcounts $l] 0] != "0.0.0.XXX"} {
      puts "[lindex [array get ipcounts $l] 0]\t\t[lindex [array get ipcounts $l] 1]\t$precom\.$postcom%"
    } else {
      puts "Unknown\t\t\t[lindex [array get ipcounts $l] 1]\t$precom\.$postcom%"
    }
  }
}

proc usage {} {
  global version
  puts "WLAN-stats-generator $version for Kismet's .csv-files"
  puts "Written 2006 by Jannis Achstetter <jannis_achstetter@web.de>\n"
  puts "USAGE:"
  puts "  ./wlanstats \[OPTIONS\] --path=PATH\n"
  puts "OPTIONS:"
  puts "  Do not combine options like \"-v -l\" => \"-vl\" as it won'\t work!"
  puts "  -v\t\tBe verbose"
  puts "  -l\t\tList all networks (You will need 165 COLs,"
  puts "  \t\telse text will be wrapped)"
  puts "  --path=\tspecifies the path where the .csv-files are stored\n"
  puts "EXAMPLE: ./wlanstats -v --path=/tmp/"
  exit 0
}

if {$::argc == 0} {usage}

set verbose 0
set listnws 0
foreach m $::argv {
  if {$m == "-v"} {set verbose 1}
  if {[lindex [split $m =] 0] == "--path"} {set path [lindex [split $m =] 1]}
  if {$m == "-l"} {set listnws 1}
  if {$m == "-?"} {usage}
  if {$m == "-h"} {usage}
  if {$m == "--help"} {usage}
}

if {[catch {string length $path}]} {
  puts "You didn't specify a path to search the .csv-files in!"
  usage
}

# We need the trailing "/" at the end of the path, so we append it if it ain't there:
if {[string index $path end] != "/"} {append path "/"}

scanfiles

exit 0
